#include "../ast3d.h"

static const AstFixedpoint_t max_velocity = AstConvertToFix( .75 );

/*
** AstAsteroid::AstAsteroid()
**
** Constructor for the AstAsteroid class.  Initializes various
** variables, and randomly selects starting position and
** rotation speed.
*/
AstAsteroid::AstAsteroid( void ) : AstDrifter( "Asteroid" )
{
   /*
   ** Set asteroid's position, rotation speed, and velocity
   */
   AstVector start_position;

   int a = PLAY_FIELD_WIDTH / 2 - random( PLAY_FIELD_WIDTH );
   int b = PLAY_FIELD_WIDTH / 2 - random( PLAY_FIELD_WIDTH );
   int c = PLAY_FIELD_WIDTH / 2 - random( PLAY_FIELD_WIDTH );

   start_position.X = AstConvertToFix( a );
   start_position.Y = AstConvertToFix( b );
   start_position.Z = AstConvertToFix( c );
   MoveTo( start_position );

   float va = random( 100 ) / 300.0 - .15;
   float vb = random( 100 ) / 300.0 - .15;
   float vc = random( 100 ) / 300.0 - .15;
   velocity.X = AstConvertToFix( va );
   velocity.Y = AstConvertToFix( vb );
   velocity.Z = AstConvertToFix( vc );

   rotation.X = random( 200 ) - 100;
   rotation.Y = random( 200 ) - 100;
   rotation.Z = random( 200 ) - 100;
   RotateAroundX( ( unsigned short ) ( rotation.X * 10 ) );
   RotateAroundY( ( unsigned short ) ( rotation.Y * 10 ) );
   RotateAroundZ( ( unsigned short ) ( rotation.Z * 10 ) );

   /*
   ** Set appropriate flags
   */
   SetFlag( ofVisible, true );
   SetFlag( ofSolid, true );
}

/*
** void AstAsteroid::DoSomething( void )
**
** This is the asteroid's "DoSomething()" function.  As you   
** tell, they're a pretty boring lot doing nothing but spinning
** about in 3 dimensions and drifting.
*/
void AstAsteroid::DoSomething()
{
   RotateAroundX( ( unsigned short ) rotation.X );
   RotateAroundY( ( unsigned short ) rotation.Y );
   RotateAroundZ( ( unsigned short ) rotation.Z );
   Drift();
}

/*
** void AstAsteroid::YouHit( AstObject *object )
**
** This is the virtual function executed whenever the asteroid
** hits another object.  Since anything else that hits the
** asteroid will be an AstDrifter, the typecasting is valid.
** HOWEVER, if you add more objects to the game that aren't
** derived from AstDrifter, this WILL BREAK.  This is one of
** those situations where Run Time Type Identification would be
** very handy.
*/
void AstAsteroid::YouHit( AstObject *object )
{
AstVector   v = ( ( AstDrifter * ) object )->GetVelocity();
AstVector  dv;

   /*
   ** Compute the difference in velocities between the two
   ** objects and then add this to this object's velocity.
   */
   dv        = v;
   dv       -= velocity;
   velocity += dv;

   /*
   ** To make sure that we don't have asteroids sticking to each
   ** other add a random amount to each element of the velocity
   ** vector so that we have some 'bounce'.
   */
   velocity.X += AstConvertToFix( .50 ) - AstFixDivFix( AstConvertToFix( random( 100 ) ), AstConvertToFix( 100 ) );
   velocity.Y += AstConvertToFix( .50 ) - AstFixDivFix( AstConvertToFix( random( 100 ) ), AstConvertToFix( 100 ) );
   velocity.Z += AstConvertToFix( .50 ) - AstFixDivFix( AstConvertToFix( random( 100 ) ), AstConvertToFix( 100 ) );

   /*
   ** Make sure that the overall magnitude of the asteroid's
   ** velocity doesn't exceed some arbitrary max.
   */
   AstFixedpoint_t magnitude = velocity.Magnitude();

   if ( magnitude > max_velocity )
   {
      velocity.X = AstFixDivFix( AstFixMulFix( max_velocity, velocity.X ), magnitude );
      velocity.Y = AstFixDivFix( AstFixMulFix( max_velocity, velocity.Y ), magnitude );
      velocity.Z = AstFixDivFix( AstFixMulFix( max_velocity, velocity.Z ), magnitude );
   }

   /*
   ** Start spinning in a new direction so that being hit is
   ** a bit more obvious.
   */
   rotation.X = random( 200 ) - 100;
   rotation.Y = random( 200 ) - 100;
   rotation.Z = random( 200 ) - 100;
}
