#ifndef __AST3D_H
#define __AST3D_H

/*
** Load commonly used library header files
*/

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>


/*
** Compiler specific constructs
*/

#  define DISABLE()            disable()
#  define ENABLE()             enable()
#  define OUTPORTB(a,b)        outportb(a,b)
#  define INPORTB(a)           inportb(a)
#  define REGISTERWORD( r, a ) ( r.x.a )
#  ifdef __INLINING__
#    pragma inline
#  endif
#  define INT86( a, b, c )     int86( a, b, c )
#  define INT86X( a, b, c, d ) int86x( a, b, c, d )
inline short int random( short int val )
{
   return ( short ) (((long)rand()*val)/(RAND_MAX+1));
}
#endif

/*
** Load header files for other
*/
#include "atype.h"
#include "amath.h"
#include "apalette.h"
#include "avga.h"
#include "agamept.h"

#include "ainput.h"
#include "akbd.h"
#include "amouse.h"
#include "ajoy.h"
#include "athrustm.h"

#include "atexmap.h"
#include "atri.h"
#include "aobjdef.h"
#include "aobject.h"
#include "atrig.h"
#include "acamera.h"
#include "aworld.h"

#include "adrifter.h"
#include "amissile.h"
#include "aplayer.h"
#include "aast.h"

struct AstConfig
{
   bool joystick, rcs, wcs;
   bool texturing;
   bool mouse;
};

const int PLAY_FIELD_WIDTH = 50;

/*
** Support functions
*/
void AstEnd( void );
void AstInit( int argc,char *argv[] );
void AstLibEnd( void );
void AstLibInit( void );
void AstFatal( const char *msg );

/*
** Variable declarations
*/
const AstFixedpoint_t near_z = AstConvertToFix( 4 );

extern AstVGA       *vga;
extern AstConfig     config;
extern AstTrigTable *trig_table;

extern AstPlayer    *player;
extern AstWorld      world;
extern AstCamera     camera;

extern AstGameport  *gameport;
extern AstKeyboard  *keyboard;
extern Ast2DInputDevice *inputdevice2d;
extern AstRCS       *rcs;
extern AstWCS       *wcs;

extern const AstMatrix4x3 *world_to_camera_matrix;

