
#define		MAXDATA 65536

#define		POINTS		1
#define		WIRE		2
#define		FLAT		4
#define		GOURAUD		8
#define		ENVMAP		16
#define     TEXTURE      32

#define     ZBUFFER     256

#define     LIGHTVIEW   0
#define     LIGHTMOVE   1

#define PI 3.14151693
#define D2R 180.0/PI
#ifndef fixm
#define fixim
class Fixed {

private:

	int	g; // the guts

const static int BP= 16;  // how many low bits are right of Binary Point
const static int BP2= BP*2;  // how many low bits are right of Binary Point
const static int BPhalf= BP/2;  // how many low bits are right of Binary Point

static double STEP() { return 1.0 / (1<<BP); }  // smallest step we can represent

	// for private construction via guts
	enum FixedRaw { RAW };
	Fixed(FixedRaw, int guts) : g(guts) {}


public:
	Fixed() : g(0) {}
	Fixed(const Fixed& a) : g( a.g ) {}
	Fixed(float a) : g( int(a / (float)STEP()) ) {}
	Fixed(double a) : g( int(a / (double)STEP()) ) {}
	Fixed(int a) : g( a << BP ) {}
	Fixed(long a) : g( a << BP ) {}
	Fixed& operator =(const Fixed& a) { g= a.g; return *this; }
	Fixed& operator =(float a) { g= Fixed(a).g; return *this; }
	Fixed& operator =(double a) { g= Fixed(a).g; return *this; }
	Fixed& operator =(int a) { g= Fixed(a).g; return *this; }
	Fixed& operator =(long a) { g= Fixed(a).g; return *this; }

	operator float() { return g * (float)STEP(); }
	operator double() { return g * (double)STEP(); }
	operator int() { return g>>BP; }
//	operator Sint16() { return g>>BP; }
	operator long() { return g>>BP; }

	Fixed operator +() const { return Fixed(RAW,g); }
	Fixed operator -() const { return Fixed(RAW,-g); }

	Fixed operator +(const Fixed& a) const { return Fixed(RAW, g + a.g); }
	Fixed operator -(const Fixed& a) const { return Fixed(RAW, g - a.g); }	
#if 1
	// more acurate, using long long
	Fixed operator *(const Fixed& a) const { return Fixed(RAW,  (int)( ((long long)g * (long long)a.g ) >> BP)); }
#else
	// faster, but with only half as many bits right of binary point
	Fixed operator *(const Fixed& a) const { return Fixed(RAW, (g>>BPhalf) * (a.g>>BPhalf) ); }
#endif
	Fixed operator /(const Fixed& a) const { return Fixed(RAW, int( (((long long)g << BP2) / (long long)(a.g)) >> BP) ); }

	Fixed operator +(float a) const { return Fixed(RAW, g + Fixed(a).g); }
	Fixed operator -(float a) const { return Fixed(RAW, g - Fixed(a).g); }
	Fixed operator *(float a) const { return Fixed(RAW, (g>>BPhalf) * (Fixed(a).g>>BPhalf) ); }
	Fixed operator /(float a) const { return Fixed(RAW, int( (((long long)g << BP2) / (long long)(Fixed(a).g)) >> BP) ); }
	
	Fixed operator +(int a) const { return Fixed(RAW, g + Fixed(a).g); }
	Fixed operator -(int a) const { return Fixed(RAW, g - Fixed(a).g); }
	Fixed operator *(int a) const { return Fixed(RAW, (g>>BPhalf) * (Fixed(a).g>>BPhalf) ); }
	Fixed operator /(int a) const { return Fixed(RAW, int( (((long long)g << BP2) / (long long)(Fixed(a).g)) >> BP) ); }
	
	Fixed operator +(long a) const { return Fixed(RAW, g + Fixed(a).g); }
	Fixed operator -(long a) const { return Fixed(RAW, g - Fixed(a).g); }
	Fixed operator *(long a) const { return Fixed(RAW, (g>>BPhalf) * (Fixed(a).g>>BPhalf) ); }
	Fixed operator /(long a) const { return Fixed(RAW, int( (((long long)g << BP2) / (long long)(Fixed(a).g)) >> BP) ); }

	Fixed operator +(double a) const { return Fixed(RAW, g + Fixed(a).g); }
	Fixed operator -(double a) const { return Fixed(RAW, g - Fixed(a).g); }
	Fixed operator *(double a) const { return Fixed(RAW, (g>>BPhalf) * (Fixed(a).g>>BPhalf) ); }
	Fixed operator /(double a) const { return Fixed(RAW, int( (((long long)g << BP2) / (long long)(Fixed(a).g)) >> BP) ); }

	Fixed& operator +=(Fixed a) { return *this = *this + a; return *this; }
	Fixed& operator -=(Fixed a) { return *this = *this - a; return *this; }
	Fixed& operator *=(Fixed a) { return *this = *this * a; return *this; }
	Fixed& operator /=(Fixed a) { return *this = *this / a; return *this; }

	Fixed& operator +=(int a) { return *this = *this + (Fixed)a; return *this; }
	Fixed& operator -=(int a) { return *this = *this - (Fixed)a; return *this; }
	Fixed& operator *=(int a) { return *this = *this * (Fixed)a; return *this; }
	Fixed& operator /=(int a) { return *this = *this / (Fixed)a; return *this; }

	Fixed& operator +=(long a) { return *this = *this + (Fixed)a; return *this; }
	Fixed& operator -=(long a) { return *this = *this - (Fixed)a; return *this; }
	Fixed& operator *=(long a) { return *this = *this * (Fixed)a; return *this; }
	Fixed& operator /=(long a) { return *this = *this / (Fixed)a; return *this; }

	Fixed& operator +=(float a) { return *this = *this + a; return *this; }
	Fixed& operator -=(float a) { return *this = *this - a; return *this; }
	Fixed& operator *=(float a) { return *this = *this * a; return *this; }
	Fixed& operator /=(float a) { return *this = *this / a; return *this; }

	Fixed& operator +=(double a) { return *this = *this + a; return *this; }
	Fixed& operator -=(double a) { return *this = *this - a; return *this; }
	Fixed& operator *=(double a) { return *this = *this * a; return *this; }
	Fixed& operator /=(double a) { return *this = *this / a; return *this; }

	bool operator ==(const Fixed& a) const { return g == a.g; }
	bool operator !=(const Fixed& a) const { return g != a.g; }
	bool operator <=(const Fixed& a) const { return g <= a.g; }
	bool operator >=(const Fixed& a) const { return g >= a.g; }
	bool operator  <(const Fixed& a) const { return g  < a.g; }
	bool operator  >(const Fixed& a) const { return g  > a.g; }

	bool operator ==(float a) const { return g == Fixed(a).g; }
	bool operator !=(float a) const { return g != Fixed(a).g; }
	bool operator <=(float a) const { return g <= Fixed(a).g; }
	bool operator >=(float a) const { return g >= Fixed(a).g; }
	bool operator  <(float a) const { return g  < Fixed(a).g; }
	bool operator  >(float a) const { return g  > Fixed(a).g; }
	
	bool operator ==(int a) const { return g == Fixed(a).g; }
	bool operator !=(int a) const { return g != Fixed(a).g; }
	bool operator <=(int a) const { return g <= Fixed(a).g; }
	bool operator >=(int a) const { return g >= Fixed(a).g; }
	bool operator  <(int a) const { return g  < Fixed(a).g; }
	bool operator  >(int a) const { return g  > Fixed(a).g; }
	
	bool operator ==(long a) const { return g == Fixed(a).g; }
	bool operator !=(long a) const { return g != Fixed(a).g; }
	bool operator <=(long a) const { return g <= Fixed(a).g; }
	bool operator >=(long a) const { return g >= Fixed(a).g; }
	bool operator  <(long a) const { return g  < Fixed(a).g; }
	bool operator  >(long a) const { return g  > Fixed(a).g; }

	bool operator ==(double a) const { return g == Fixed(a).g; }
	bool operator !=(double a) const { return g != Fixed(a).g; }
	bool operator <=(double a) const { return g <= Fixed(a).g; }
	bool operator >=(double a) const { return g >= Fixed(a).g; }
	bool operator  <(double a) const { return g  < Fixed(a).g; }
	bool operator  >(double a) const { return g  > Fixed(a).g; }
};
/*
inline Fixed operator +(float a, const Fixed& b) { return Fixed(a)+b; }
inline Fixed operator -(float a, const Fixed& b) { return Fixed(a)-b; }
inline Fixed operator *(float a, const Fixed& b) { return Fixed(a)*b; }
inline Fixed operator /(float a, const Fixed& b) { return Fixed(a)/b; }

inline bool operator ==(float a, const Fixed& b) { return Fixed(a) == b; }
inline bool operator !=(float a, const Fixed& b) { return Fixed(a) != b; }
inline bool operator <=(float a, const Fixed& b) { return Fixed(a) <= b; }
inline bool operator >=(float a, const Fixed& b) { return Fixed(a) >= b; }
inline bool operator  <(float a, const Fixed& b) { return Fixed(a)  < b; }
inline bool operator  >(float a, const Fixed& b) { return Fixed(a)  > b; }



inline Fixed operator +(double a, const Fixed& b) { return Fixed(a)+b; }
inline Fixed operator -(double a, const Fixed& b) { return Fixed(a)-b; }
inline Fixed operator *(double a, const Fixed& b) { return Fixed(a)*b; }
inline Fixed operator /(double a, const Fixed& b) { return Fixed(a)/b; }

inline bool operator ==(double a, const Fixed& b) { return Fixed(a) == b; }
inline bool operator !=(double a, const Fixed& b) { return Fixed(a) != b; }
inline bool operator <=(double a, const Fixed& b) { return Fixed(a) <= b; }
inline bool operator >=(double a, const Fixed& b) { return Fixed(a) >= b; }
inline bool operator  <(double a, const Fixed& b) { return Fixed(a)  < b; }
inline bool operator  >(double a, const Fixed& b) { return Fixed(a)  > b; }


int& operator +=(int& a, const Fixed& b) { a = (Fixed)a + b; return a; }
int& operator -=(int& a, const Fixed& b) { a = (Fixed)a - b; return a; }
int& operator *=(int& a, const Fixed& b) { a = (Fixed)a * b; return a; }
int& operator /=(int& a, const Fixed& b) { a = (Fixed)a / b; return a; }

long& operator +=(long& a, const Fixed& b) { a = (Fixed)a + b; return a; }
long& operator -=(long& a, const Fixed& b) { a = (Fixed)a - b; return a; }
long& operator *=(long& a, const Fixed& b) { a = (Fixed)a * b; return a; }
long& operator /=(long& a, const Fixed& b) { a = (Fixed)a / b; return a; }

float& operator +=(float& a, const Fixed& b) { a = a + b; return a; }
float& operator -=(float& a, const Fixed& b) { a = a - b; return a; }
float& operator *=(float& a, const Fixed& b) { a = a * b; return a; }
float& operator /=(float& a, const Fixed& b) { a = a / b; return a; }

double& operator +=(double& a, const Fixed& b) { a = a + b; return a; }
double& operator -=(double& a, const Fixed& b) { a = a - b; return a; }
double& operator *=(double& a, const Fixed& b) { a = a * b; return a; }
double& operator /=(double& a, const Fixed& b) { a = a / b; return a; }*/
#endif
typedef struct vector3d
{
    int x;
    int y;
    int z;
} vector3d;

typedef struct point2d
{
	int x;
	int y;
	unsigned short c;
} point2d;

typedef struct tcord
{
    int u, v;
} tcord;

typedef struct point3d
{
	int x;
	int y;
	int z;
	tcord t;
	unsigned short c;
} point3d;


typedef struct poly2d
{
	int p0; tcord tc0;
	int p1; tcord tc1;
	int p2; tcord tc2;
	int c;
    int m;
} poly2d;


typedef struct line2d
{
	int p0;
	int p1;
	int c;
}line2d;


typedef struct rot3d
{
	float x;
	float y;
	float z;
}rot3d;


typedef struct pos3d
{
	float x;
	float y;
	float z;
}pos3d;


typedef struct object3d
{
    int npts, npls, nlns;

    point3d *point;
    poly2d *poly;
    line2d *line;

    vector3d *normal;
    vector3d *pt_normal;

    rot3d rot;
    pos3d pos;
}object3d;

vector3d CrossProduct(vector3d v1, vector3d v2);
int DotProduct(vector3d v1, vector3d v2);
vector3d Normalize(vector3d v);
vector3d NegVec(vector3d v);

void Calc3d(object3d *obj);

void rotate3d (object3d *obj);
void translate3d (object3d *obj);
void project3d (object3d *obj);

void rotate3d_normals (object3d *obj);
void rotate3d_pt_normals (object3d *obj);

void CalcPointColor(object3d *obj);
void CalcPolyColor(object3d *obj);

void quicksort (int lo, int hi, int data[]);
